@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2010 Rochet Sylvain
+ *  Copyright © 2002-2014 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,13 +19,8 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#define GETDATA_STEP 65536
-#define SECURITY_SIZE 1024
-
-
 /* ** prototypes for connect/disconnect to server */
 gboolean client_connect(gint32 *idconnect, gchar* host, gint32 port);
 gboolean client_disconnect(gint32 id);
-gboolean client_fonct();
-gint32   client_recv(gint32 id);
+gboolean client_event(GIOChannel *channel, GIOCondition condition, gpointer data);
 gboolean client_send(gint32 id, gchar *data, gint32 lenght_data);
