@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2010 Rochet Sylvain
+ *  Copyright © 2002-2014 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,18 +21,12 @@
 
 #include <gtk/gtk.h>
 
-#define MAX_ROW_LENGTH_TITLE_ESTATE 23
-
+void display_init();
 void display_hide();
-void display_unhide();
-void display_clear_all();
-void display_clear_title();
+void display_show();
 void display_clear_text();
 void display_clear_buttons();
-void display_init();
 void display_estate(gint32 estateid);
 void display_title(gchar *text);
 void display_text(gchar *message);
-void display_button(gchar *caption, gchar *command, gboolean enabled);
-
-
+void display_add_button(gchar *caption, gchar *command, gboolean enabled);
