@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2010 Rochet Sylvain
+ *  Copyright © 2002-2014 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,8 +19,24 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+
 /* all includes of engine */
 
+#ifndef _ENG_H
+#define _ENG_H
+
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
+
 #include "eng_main.h"
-#include "eng_conv.h"
 #include "eng_frame.h"
+
+
+#ifdef __cplusplus
+}
+#endif
+#endif /* _ENG_H */
+
