@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2010 Rochet Sylvain
+ *  Copyright © 2002-2014 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -105,22 +105,28 @@ gboolean create_home_directory()  {
 void read_config_files()   {
 
 	gchar *filename;
-	struct stat s;
 
-	/* read default config file */
-	filename = g_strconcat(PACKAGE_DATA_DIR, "/default.conf", NULL);
-	if(! parse_file_config(filename) ) {
-		printf("Error opening config file %s\n",filename);
-		exit(-1);
-	}
+	/* Default configuration */
+	config->nickname = g_strdup("anonymous");
 
-	g_free(filename);
+	config->metaserver_autogetlist = FALSE;
+	config->metaserver_autogetgames = TRUE;
+	config->metaserver_sendclientversion = TRUE;
+
+	config->getgames_host = g_strdup("localhost");
+	config->getgames_port = 1234;
+	config->getgames_autogetgames = TRUE;
+
+	config->message_max_lines = 40;
+	config->chat_max_lines = 80;
+
+	config->game_playerlist_position = GAME_PLAYERLIST_POS_LEFT;
+	config->game_token_animation_speed = 300;
+	config->game_token_transparency = FALSE;
 
 	/* read user config file */
 	filename = g_strconcat(global->path_home, "gtkatlantic.conf", NULL);
-	if(stat(filename, &s) >= 0)
-		parse_file_config(filename);
-
+	parse_file_config(filename);
 	g_free(filename);
 }
 
@@ -148,8 +154,15 @@ void create_connection_metaserver(gint32 gametype)  {
 	gint32 connectid;
 	gchar *sendstr;
 
-	if(! client_connect(&connectid, config->metaserver_host, config->metaserver_port) ) return;
+	/* close previously opened connection */
+	if (global->metaserver_connectid) {
+		client_disconnect(global->metaserver_connectid);
+		global->metaserver_connectid = 0;
+	}
+
+	if(! client_connect(&connectid, METASERVER_HOST, METASERVER_PORT) ) return;
 	connection[connectid]->type = gametype;
+	global->metaserver_connectid = connectid;
 
 	/* send client version (to retrieve info about release) */
 	if(config->metaserver_sendclientversion)  {
@@ -172,16 +185,43 @@ void create_connection_metaserver(gint32 gametype)  {
 }
 
 
-/* this function made a new connection for get games only */
+/* this function create a new connection to get games list */
 void create_connection_get_games(gchar *host, gint32 port)  {
 
 	gint32 connectid;
 	gchar *sendstr;
+	GtkListStore *store;
+
+	/* close previously opened connection */
+	if (global->customserver_connectid) {
+		client_disconnect(global->customserver_connectid);
+		global->customserver_connectid = 0;
+	}
+
+	/* remove all games previously fetched from custom server */
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(global->GameList)));
+	if (store) {
+		GtkTreeIter iter;
+		gboolean valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
+		while (valid) {
+			gint32 source;
+			GtkTreeIter curiter = iter;
+
+			valid = gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter); /* get next iter so we can remove the current entry safely */
+
+			gtk_tree_model_get(GTK_TREE_MODEL(store), &curiter, GAMELIST_COLUMN_SOURCE, &source, -1);
+			if (source == GAMELIST_SOURCE_CUSTOM)  {
+				gtk_list_store_remove(store, &curiter);
+			}
+		}
+	}
 
 	if(! client_connect(&connectid, host, port) ) return;
 	connection[connectid]->type = CONNECT_TYPE_MONOPD_GETGAME;
+	global->customserver_connectid = connectid;
 
-	sendstr = g_strdup(".n_client_\n.gl\n");
+	/* FIXME: wait for server banner */
+	sendstr = g_strdup(".n_client_\n");
 	client_send(connectid, sendstr, strlen(sendstr));
 	g_free(sendstr);
 }
@@ -200,8 +240,10 @@ void text_insert_message(gchar* text, gint32 length)  {
 
 	textbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(global->Message));
 	gtk_text_buffer_get_end_iter(textbuff, &textiter);
+	if (gtk_text_iter_get_offset(&textiter)) {
+		gtk_text_buffer_insert(textbuff, &textiter, "\n", -1);
+	}
 	gtk_text_buffer_insert(textbuff, &textiter, text, length);
-	gtk_text_buffer_insert(textbuff, &textiter, "\n", -1);
 
 	/* Scroll the text */
 	textmark = gtk_text_buffer_get_mark(textbuff, "endmark");
@@ -259,92 +301,127 @@ void text_insert_chat(gchar* text, gint32 length)
 	if(global->phase < PHASE_GAMECREATE)
 		return;
 
-	textbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(game->Chat));
+	textbuff = gtk_text_view_get_buffer(GTK_TEXT_VIEW(currentgame->Chat));
 	gtk_text_buffer_get_end_iter(textbuff, &textiter);
+	if (gtk_text_iter_get_offset(&textiter)) {
+		gtk_text_buffer_insert(textbuff, &textiter, "\n", -1);
+	}
 	gtk_text_buffer_insert(textbuff, &textiter, text, length);
-	gtk_text_buffer_insert(textbuff, &textiter, "\n", -1);
 
 	/* Scroll to the end mark */
 	textmark = gtk_text_buffer_get_mark(textbuff, "endmark");
-	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(game->Chat), textmark,
+	gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(currentgame->Chat), textmark,
 			0.0, FALSE, 0.0, 0.0);
 
-	game->chat_nb_lines++;
+	currentgame->chat_nb_lines++;
 
-/*	while(game->chat_nb_lines > config->chat_max_lines) {
+/*	while(currentgame->chat_nb_lines > config->chat_max_lines) {
 
 		// if unlimited
 		if(config->chat_max_lines <= 0)  break;
 
-		len = gtk_text_get_length( GTK_TEXT(game->Chat) );
+		len = gtk_text_get_length( GTK_TEXT(currentgame->Chat) );
 
-		for(i = 0 ;  GTK_TEXT_INDEX( GTK_TEXT(game->Chat), i) != '\n' ; i++)  {
+		for(i = 0 ;  GTK_TEXT_INDEX( GTK_TEXT(currentgame->Chat), i) != '\n' ; i++)  {
 
 			if(i >= len)  { // argl... no CR found
 
-				gtk_text_set_point( GTK_TEXT(game->Chat), 0);
-				gtk_text_forward_delete( GTK_TEXT(game->Chat), len);
-				game->chat_nb_lines = 0;
+				gtk_text_set_point( GTK_TEXT(currentgame->Chat), 0);
+				gtk_text_forward_delete( GTK_TEXT(currentgame->Chat), len);
+				currentgame->chat_nb_lines = 0;
 				break;
 			}
 		}
 
-		if(game->chat_nb_lines)  {
+		if(currentgame->chat_nb_lines)  {
 
 			i++;
-			gtk_text_set_point( GTK_TEXT(game->Chat), 0);
-			gtk_text_forward_delete( GTK_TEXT(game->Chat), i);
-			game->chat_nb_lines--;
+			gtk_text_set_point( GTK_TEXT(currentgame->Chat), 0);
+			gtk_text_forward_delete( GTK_TEXT(currentgame->Chat), i);
+			currentgame->chat_nb_lines--;
 		}
 	}
 
-	len = gtk_text_get_length( GTK_TEXT(game->Chat) );
-	gtk_text_set_point( GTK_TEXT(game->Chat), len);
+	len = gtk_text_get_length( GTK_TEXT(currentgame->Chat) );
+	gtk_text_set_point( GTK_TEXT(currentgame->Chat), len);
 
-	gtk_text_thaw( GTK_TEXT(game->Chat) );
+	gtk_text_thaw( GTK_TEXT(currentgame->Chat) );
 
-//	gtk_adjustment_set_value( GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj ),  GTK_ADJUSTMENT( GTK_TEXT( game->Chat )->vadj )->upper );
+//	gtk_adjustment_set_value( GTK_ADJUSTMENT( GTK_TEXT( currentgame->Chat )->vadj ),  GTK_ADJUSTMENT( GTK_TEXT( currentgame->Chat )->vadj )->upper );
 
 	*/
 }
 
 
-/* alloc all memory for a game */
-void game_alloc()  {
+game *game_new(gint32 id) {
+	game *game;
 
-	game = g_malloc0(sizeof (_game) );
+	game = g_malloc0(sizeof(struct game_s));
+	game->gameid = id;
+	printf("New game: %d\n", game->gameid);
+
+	game->next = game_h;
+	game_h = game;
+	return game;
 }
 
 
-/* free entirely a game, return to get games page */
-void game_free()  {
+void game_free(gint32 id) {
+	game **prevp, *game;
 
+	for (prevp = &game_h; (game = *prevp); prevp = &game->next) {
+		if (game->gameid == id) {
+			*prevp = game->next;
+			printf("Deleted game: %d\n", game->gameid);
+			g_free(game);
+			break;
+		}
+	}
+}
+
+
+game *game_find(gint32 id) {
+	game *game;
+
+	for (game = game_h; game; game = game->next) {
+		if (game->gameid == id) {
+			return game;
+		}
+	}
+	return NULL;
+}
+
+
+/* free all game ressources, return to get games page */
+void game_quit()  {
 	gint32 i;
+	game *game, *cur;
 
-	client_disconnect(game->connectid);
+	client_disconnect(global->game_connectid);
+	global->game_connectid = 0;
 
 	interface_destroy_message();
 	interface_destroy_chat();
-	if(game->WinEstateTree)  gtk_widget_destroy(game->WinEstateTree);
+	if(currentgame->WinEstateTree)  gtk_widget_destroy(currentgame->WinEstateTree);
 
-	if(game->timeout_token)  gtk_timeout_remove(game->timeout_token);
+	if(currentgame->timeout_token)  g_source_remove(currentgame->timeout_token);
 
 	interface_create_getgamespage();
 
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
 		if(! global->player[i].playerid) continue;
-		game_delete_player(i);
+		game_free_player(i);
 	}
 
 	for(i = 0 ; i < data->number_estates ; i++)  {
 
-		if(game->estate[i].name) g_free(game->estate[i].name);
+		if(currentgame->estate[i].name) g_free(currentgame->estate[i].name);
 	}
 
 	for(i = 0 ; i < MAX_GROUPS ; i++)  {
 
-		if(game->group[i].name) g_free(game->group[i].name);
+		if(currentgame->group[i].name) g_free(currentgame->group[i].name);
 	}
 
 	for(i = 0 ; i < MAX_TRADES ; i++)  {
@@ -354,32 +431,42 @@ void game_free()  {
 
 	for(i = 0 ; i < MAX_COMMANDS ; i++)  {
 
-		if(game->command[i].open  &&  game->command[i].command)
-			g_free(game->command[i].command);
+		if(currentgame->command[i].open  &&  currentgame->command[i].command)
+			g_free(currentgame->command[i].command);
 	}
 
-	frame_destroy( game->board_frame );
+	eng_frame_destroy( currentgame->board_frame );
 
 	game_free_pngs();
 
-	g_free(game);
-	game = 0;
+	/* remove all games */
+	game = game_h;
+	while (game) {
+		cur = game;
+		game = game->next;
+		printf("Deleted game: %d\n", cur->gameid);
+		g_free(cur);
+	}
+	game_h = NULL;
+
+	currentgame = NULL;
 }
 
 
-/* return a valid game id */
-gboolean game_get_valid_player_slot(gint32 *playerslot)  {
+/* create a new player slot */
+gint32 game_new_player_slot(gint32 playerid)  {
 
 	gint32 i;
 
-	for(i = 0; i < MAX_PLAYERS; i++)
-		if(!global->player[i].playerid) {
-
-			*playerslot = i;
-			return TRUE;
+	for (i = 0; i < MAX_PLAYERS; i++) {
+		if (!global->player[i].playerid) {
+			memset(&(global->player[i]), 0, sizeof(_player));
+			global->player[i].playerid = playerid;
+			return i;
 		}
+	}
 
-	return FALSE;
+	return -1;
 }
 
 
@@ -424,7 +511,7 @@ gboolean game_get_valid_command_slot(gint32 *commandslot)  {
 	gint32 i;
 
 	for(i = 0; i < MAX_COMMANDS ; i++)
-		if(!game->command[i].open) {
+		if(!currentgame->command[i].open) {
 
 			*commandslot = i;
 			return TRUE;
@@ -442,9 +529,9 @@ gint32 get_command_button_slot_with_command(gchar *command) {
 
 	for(i = 0 ; i < MAX_COMMANDS ; i ++)  {
 
-		if(game->command[i].open)  {
+		if(currentgame->command[i].open)  {
 
-			if(! strncmp(command, game->command[i].command, strlen(command)) )
+			if(! strncmp(command, currentgame->command[i].command, strlen(command)) )
 				return i;
 		}
 	}
@@ -462,7 +549,7 @@ void parse_specific_chat_message(gchar *message)   {
 	struct timeval tv;
 	time_t t_now;
 
-	pslot = get_player_slot_with_playerid( game->my_playerid );
+	pslot = get_player_slot_with_playerid( global->my_playerid );
 
 	if(message[0] != '!') return;
 
@@ -481,7 +568,7 @@ void parse_specific_chat_message(gchar *message)   {
 		if(strlen(msg) == 0  ||  !strcmp(msg, global->player[pslot].name) )  {
 
 			sendstr = g_strdup_printf("GtkAtlantic %s\n", VERSION);
-			client_send(game->connectid, sendstr, strlen(sendstr));
+			client_send(global->game_connectid, sendstr, strlen(sendstr));
 			g_free(sendstr);
 		}
 	}
@@ -501,7 +588,7 @@ void parse_specific_chat_message(gchar *message)   {
 			gettimeofday(&tv, NULL);
 			t_now = tv.tv_sec;
 			sendstr = g_strdup_printf("%s", ctime(&t_now) );
-			client_send(game->connectid, sendstr, strlen(sendstr));
+			client_send(global->game_connectid, sendstr, strlen(sendstr));
 			g_free(sendstr);
 		}
 	}
@@ -518,7 +605,7 @@ void parse_specific_chat_message(gchar *message)   {
 		if(strlen(msg) == 0  ||  !strcmp(msg, global->player[pslot].name) )  {
 
 			sendstr = g_strdup_printf("pong\n");
-			client_send(game->connectid, sendstr, strlen(sendstr));
+			client_send(global->game_connectid, sendstr, strlen(sendstr));
 			g_free(sendstr);
 		}
 	}
@@ -554,7 +641,8 @@ void parse_specific_send_message(gchar *message)   {
 			for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
 				if(! global->player[i].playerid)  continue;
-				if(global->player[i].game != game->gameid)  continue;
+				if (global->player[i].spectator) continue;
+				if(global->player[i].game != currentgame->gameid)  continue;
 
 				tmp = g_strdup_printf("<%s> %d", global->player[i].name, game_get_assets_player(global->player[i].playerid) );
 				text_insert_message(tmp, strlen(tmp) );
@@ -582,10 +670,10 @@ void parse_specific_send_message(gchar *message)   {
 			else  msg++;
 		}
 
-		if(strlen(msg) != 0  &&  strcmp(global->player[ get_player_slot_with_playerid(game->my_playerid) ].name, msg) )  {
+		if(strlen(msg) != 0  &&  strcmp(global->player[ get_player_slot_with_playerid(global->my_playerid) ].name, msg) )  {
 
 			tmp = g_strdup_printf(".n%s\n", msg);
-			client_send(game->connectid, tmp, strlen(tmp));
+			client_send(global->game_connectid, tmp, strlen(tmp));
 			g_free(tmp);
 		}
 	}
@@ -614,10 +702,10 @@ void parse_specific_send_message(gchar *message)   {
 
 	else if(! strncmp(msg, "elapsed", 7))  {
 
-		if(game->status >= GAME_STATUS_RUN)  {
+		if(currentgame->status >= GAME_STATUS_RUN)  {
 
 			gettimeofday(&tv, NULL);
-			t_now = tv.tv_sec - game->start_time;
+			t_now = tv.tv_sec - currentgame->start_time;
 		}
 		else  t_now = 0;
 
@@ -640,14 +728,14 @@ void parse_specific_send_message(gchar *message)   {
 		if(strlen(msg) != 0)  {
 
 			tmp = g_strdup_printf("[ACTION] %s\n", msg);
-			client_send(game->connectid, tmp, strlen(tmp));
+			client_send(global->game_connectid, tmp, strlen(tmp));
 			g_free(tmp);
 		}
 	}
 }
 
 
-/* sort playerlist by playerid (ascending) */
+/* sort playerlist by playerid (ascending) */   /* FIXME: remove that */
 void game_sort_playerlist_by_playerid()  {
 
 	gint32 i, j;
@@ -663,17 +751,52 @@ void game_sort_playerlist_by_playerid()  {
 
 			if(global->player[j].playerid < global->player[i].playerid)  {
 
-				memcpy(&playertmp, &global->player[i], sizeof(_player) );
-				memcpy(&global->player[i], &global->player[j], sizeof(_player) );
-				memcpy(&global->player[j], &playertmp, sizeof(_player) );
+				memcpy(&playertmp, &(global->player[i]), sizeof(_player) );
+				memcpy(&(global->player[i]), &(global->player[j]), sizeof(_player) );
+				memcpy(&(global->player[j]), &playertmp, sizeof(_player) );
 			}
 		}
 	}
 }
 
 
+void game_switch_status() {
+	guint32 i;
+
+	switch (currentgame->status) {
+		case GAME_STATUS_CONFIG:
+			interface_create_gameconfigpage();
+			break;
+
+		case GAME_STATUS_INIT:
+			game_write_cookie();
+			interface_create_gameinitpage();
+			if( game_load_pngs() ) exit(-1);
+			break;
+	
+		case GAME_STATUS_RUN:
+			interface_create_gameboardpage();
+			break;
+
+		case GAME_STATUS_END:
+			/* remove all commands */
+			if(currentgame->CommandsBox)  gtk_widget_destroy(currentgame->CommandsBox);
+			currentgame->CommandsBox = NULL;
+
+			/* destroy all open trades */
+			for(i = 0 ; i < MAX_TRADES ; i++) {
+				trade_destroy_slot(i);
+			}
+			break;
+			
+		default:
+			return;
+	}
+}
+
+
 /* write cookie on disk */
-void game_write_cookie(gchar *cookie)  {
+void game_write_cookie()  {
 
 	gchar *filename, *errormsg;
 	FILE *file;
@@ -689,9 +812,9 @@ void game_write_cookie(gchar *cookie)  {
 		return;
 	}
 
-	fprintf(file, "%s\n", connection[ game->connectid ]->host );
-	fprintf(file, "%d\n", connection[ game->connectid ]->port );
-	fprintf(file, "%s\n", cookie);
+	fprintf(file, "%s\n", connection[ global->game_connectid ]->host);
+	fprintf(file, "%d\n", connection[ global->game_connectid ]->port);
+	fprintf(file, "%s\n", global->cookie);
 
 	g_free(filename);
 	fclose(file);
@@ -704,7 +827,7 @@ gboolean game_get_valid_card_slot(gint32 *cardslot)  {
 	gint32 i;
 
 	for(i = 0; i < MAX_CARDS ; i++)
-		if(!game->card[i].owner) {
+		if(!currentgame->card[i].owner) {
 
 			*cardslot = i;
 			return TRUE;
@@ -721,7 +844,7 @@ gint32 get_card_slot_with_cardid(gint32 cardid)  {
 
 	for(i = 0 ; i < MAX_CARDS ; i++)  {
 
-		if(game->card[i].cardid == cardid) {
+		if(currentgame->card[i].cardid == cardid) {
 
 			return(i);
 			break;
@@ -762,7 +885,8 @@ gint32 game_nb_players() {
 	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
 
 		if(!global->player[i].playerid)  continue;
-		if(global->player[i].game != game->gameid)  continue;
+		if (global->player[i].spectator) continue;
+		if(global->player[i].game != currentgame->gameid)  continue;
 		c++;
 	}
 	return c;
@@ -774,7 +898,7 @@ gboolean game_get_valid_trade_slot(gint32 *tradeslot)  {
 	gint32 i;
 
 	for(i = 0; i < MAX_TRADES ; i++)
-		if(! game->trade[i].open) {
+		if(! currentgame->trade[i].open) {
 
 			*tradeslot = i;
 			return TRUE;
@@ -791,9 +915,9 @@ gint32 get_trade_slot_with_tradeid(gint32 tradeid)  {
 
 	for(i = 0 ; i < MAX_TRADES ; i++)  {
 
-		if(! game->trade[i].open) continue;
+		if(! currentgame->trade[i].open) continue;
 
-		if(game->trade[i].tradeid == tradeid) {
+		if(currentgame->trade[i].tradeid == tradeid) {
 
 			return(i);
 			break;
@@ -812,7 +936,7 @@ gint32 get_estateid_by_estatename(gchar *name)  {
 
 	for(i = 0 ; i < data->number_estates ; i++)  {
 
-		if(! strcmp(game->estate[i].name, name) )  {
+		if(! strcmp(currentgame->estate[i].name, name) )  {
 
 			return(i);
 			break;
@@ -826,29 +950,33 @@ gint32 get_estateid_by_estatename(gchar *name)  {
 /* build player list in game config phase */
 void game_buildplayerlist()   {
 
-	gint32 i, row;
-	gchar *txt[10];
+	if (!currentgame) {
+		return;
+	}
 
-	game_sort_playerlist_by_playerid();
+	if (currentgame->status < GAME_STATUS_RUN) {
+		game_sort_playerlist_by_playerid();
+	}
 
 	/* build player list */
-	if(global->phase == PHASE_GAMECREATE)  {
+	if (global->phase == PHASE_GAMECREATE) {
+		GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(currentgame->PlayerList)));
+		GtkTreeIter iter;
+		gint32 i;
 
-		gtk_clist_freeze(GTK_CLIST(game->PlayerCList));
-		gtk_clist_clear(GTK_CLIST(game->PlayerCList));
+		gtk_list_store_clear(store);
 
 		for(i = 0 ; i < MAX_PLAYERS ; i++)  {
-
-			if(global->player[i].playerid && global->player[i].game == game->gameid)  {
-
-				txt[0] = global->player[i].name;
-				txt[1] = global->player[i].host;
-				row = gtk_clist_append(GTK_CLIST(game->PlayerCList), txt);
-				gtk_clist_set_selectable(GTK_CLIST(game->PlayerCList), row ,FALSE);
-			}
+			if (! global->player[i].playerid)  continue;
+			if (global->player[i].spectator) continue;
+			if (global->player[i].game != currentgame->gameid)  continue;
+		
+			gtk_list_store_append(store, &iter);
+			gtk_list_store_set(store, &iter,
+			PLAYERLIST_COLUMN_NAME, global->player[i].name,
+			PLAYERLIST_COLUMN_HOST, global->player[i].host,
+			-1);
 		}
-
-		gtk_clist_thaw(GTK_CLIST(game->PlayerCList));
 	}
 }
 
@@ -869,16 +997,16 @@ gint32 game_get_assets_player(gint32 playerid)  {
 	/* estates + houses */
 	for(i = 0 ; i < data->number_estates ; i++)  {
 
-		if(game->estate[i].owner != playerid)  continue;
+		if(currentgame->estate[i].owner != playerid)  continue;
 
 		/* estates */
-		if(! game->estate[i].mortgaged)
-			assets += game->estate[i].mortgageprice;
+		if(! currentgame->estate[i].mortgaged)
+			assets += currentgame->estate[i].mortgageprice;
 
 		/* houses */
-		if(game->estate[i].houses <= 0)  continue;
+		if(currentgame->estate[i].houses <= 0)  continue;
 
-		assets += game->estate[i].houses * game->estate[i].sellhouseprice;
+		assets += currentgame->estate[i].houses * currentgame->estate[i].sellhouseprice;
 	}
 
 	return assets;
@@ -899,7 +1027,8 @@ void game_update_tokens()  {
 		for(j = 0, nb_tokens = 0 ; j < MAX_PLAYERS ; j++)  {
 
 			if(! global->player[j].playerid)  continue;
-			if(global->player[j].game != game->gameid)  continue;
+			if(global->player[j].spectator) continue;
+			if(global->player[j].game != currentgame->gameid)  continue;
 			if(global->player[j].bankrupt)  continue;
 			if(global->player[j].jailed) continue;
 			if(global->player[j].location != i)  continue;
@@ -913,7 +1042,8 @@ void game_update_tokens()  {
 		for(j = 0, k = 0, xp = -1, yp = -1, pp = -1 ; j < MAX_PLAYERS ; j++)  {
 
 			if(! global->player[j].playerid)  continue;
-			if(global->player[j].game != game->gameid)  continue;
+			if(global->player[j].spectator) continue;
+			if(global->player[j].game != currentgame->gameid)  continue;
 			if(global->player[j].bankrupt)  continue;
 			if(global->player[j].jailed) continue;
 			if(global->player[j].location != i)  continue;
@@ -925,8 +1055,8 @@ void game_update_tokens()  {
 
 			if(nb_tokens == 1)  {
 
-				pic_set_x(game->board_frame, global->player[j].token_pic, x1);
-				pic_set_y(game->board_frame, global->player[j].token_pic, y1);
+				eng_pic_set_x(global->player[j].token_pic, x1);
+				eng_pic_set_y(global->player[j].token_pic, y1);
 
 				break;
 			}
@@ -949,8 +1079,8 @@ void game_update_tokens()  {
 					ys = yp - data->pngfile_token_height[ global->player[pp].buffer_token ] - 2;
 			}
 
-			pic_set_x(game->board_frame, global->player[j].token_pic, xs);
-			pic_set_y(game->board_frame, global->player[j].token_pic, ys);
+			eng_pic_set_x(global->player[j].token_pic, xs);
+			eng_pic_set_y(global->player[j].token_pic, ys);
 
 			xp = xs;
 			yp = ys;
@@ -969,7 +1099,8 @@ void game_update_tokens()  {
 		for(j = 0, nb_tokens = 0 ; j < MAX_PLAYERS ; j++)  {
 
 			if(! global->player[j].playerid)  continue;
-			if(global->player[j].game != game->gameid)  continue;
+			if(global->player[j].spectator) continue;
+			if(global->player[j].game != currentgame->gameid)  continue;
 			if(global->player[j].bankrupt)  continue;
 			if(! global->player[j].jailed) continue;
 			if(global->player[j].location != i)  continue;
@@ -983,7 +1114,8 @@ void game_update_tokens()  {
 		for(j = 0, k = 0, xp = -1, yp = -1, pp = -1 ; j < MAX_PLAYERS ; j++)  {
 
 			if(! global->player[j].playerid)  continue;
-			if(global->player[j].game != game->gameid)  continue;
+			if(global->player[j].spectator) continue;
+			if(global->player[j].game != currentgame->gameid)  continue;
 			if(global->player[j].bankrupt)  continue;
 			if(! global->player[j].jailed) continue;
 			if(global->player[j].location != i)  continue;
@@ -995,8 +1127,8 @@ void game_update_tokens()  {
 
 			if(nb_tokens == 1)  {
 
-				pic_set_x(game->board_frame, global->player[j].token_pic, x1);
-				pic_set_y(game->board_frame, global->player[j].token_pic, y1);
+				eng_pic_set_x(global->player[j].token_pic, x1);
+				eng_pic_set_y(global->player[j].token_pic, y1);
 
 				break;
 			}
@@ -1019,8 +1151,8 @@ void game_update_tokens()  {
 					ys = yp - data->pngfile_token_height[ global->player[pp].buffer_token ] - 2;
 			}
 
-			pic_set_x(game->board_frame, global->player[j].token_pic, xs);
-			pic_set_y(game->board_frame, global->player[j].token_pic, ys);
+			eng_pic_set_x(global->player[j].token_pic, xs);
+			eng_pic_set_y(global->player[j].token_pic, ys);
 
 			xp = xs;
 			yp = ys;
@@ -1030,71 +1162,86 @@ void game_update_tokens()  {
 		}
 	}
 
+	update_display();
 }
 
 
-/* start token movement */
-void game_start_move_token(gint32 playerid)  {
+/* initiate token movement */
+void game_initiate_token_movement() {
 
-	gchar *sendstr;
-	gint32 pslot;
-
-	pslot = get_player_slot_with_playerid(playerid);
-	if(pslot < 0)  return;
-
-	if(global->player[pslot].directmove)  {
-
-		global->player[pslot].location = global->player[pslot].location_to;
-		sendstr = g_strdup_printf(".t%d\n", global->player[pslot].location);
-		client_send(game->connectid, sendstr, strlen(sendstr));
-		g_free(sendstr);
+	if (global->phase != PHASE_GAMEPLAY) {
+		return;
 	}
 
-	game_update_tokens();
+	/* do nothing if token timer is running */
+	if (currentgame->timeout_token) {
+		return;
+	}
 
-	return;
+	game_move_tokens();
 }
 
 
-/* animate move token */
+/* animate token movement */
 gboolean game_move_tokens()  {
 
 	gint32 i;
 	gchar *sendstr;
-	gboolean update = 0;
+	gboolean update = FALSE, more = FALSE;
 
-	for(i = 0 ; i < MAX_PLAYERS ; i++)  {
+	for (i = 0 ; i < MAX_PLAYERS ; i++)  {
 
-		if(! global->player[i].playerid)  continue;
-		if(global->player[i].game != game->gameid)  continue;
-		if(global->player[i].location == global->player[i].location_to)  continue;
+		if (! global->player[i].playerid)  continue;
+		if (global->player[i].spectator) continue;
+		if (global->player[i].game != currentgame->gameid)  continue;
 
-		global->player[i].location++;
-		if(global->player[i].location >= data->number_estates)
-			global->player[i].location = 0;
+		if (global->player[i].directmove) {
+			global->player[i].location = global->player[i].location_to;
+			global->player[i].directmove = 0;
+		} else if (global->player[i].location != global->player[i].location_to) {
+			global->player[i].location++;
+			if (global->player[i].location >= data->number_estates) {
+				global->player[i].location = 0;
+			}
+			if (global->player[i].location != global->player[i].location_to) {
+				more = TRUE;
+			}
+		} else {
+			continue;
+		}
 
 		sendstr = g_strdup_printf(".t%d\n", global->player[i].location);
-		client_send(game->connectid, sendstr, strlen(sendstr));
+		client_send(global->game_connectid, sendstr, strlen(sendstr));
 		g_free(sendstr);
 
 		update = TRUE;
 	}
 
-	if(update)  game_update_tokens();
+	if (update) {
+		game_update_tokens();
 
-	return TRUE;
+		if (more) {
+			if (currentgame->timeout_token == 0) {
+				currentgame->timeout_token = g_timeout_add(config->game_token_animation_speed, game_move_tokens, NULL);
+			}
+			return TRUE;
+		}
+	}
+
+	currentgame->timeout_token = 0;
+	return FALSE;
 }
 
 
-void game_delete_player(gint32 id)  {
+void game_free_player(gint32 id)  {
 
 	/* delete a player */
-	frame_destroy( global->player[id].playerlist_token_frame );
-	frame_destroy( global->player[id].playerlist_cards_frame );
+	eng_frame_destroy( global->player[id].playerlist_token_frame );
+	eng_frame_destroy( global->player[id].playerlist_cards_frame );
 
 	if(global->player[id].name) g_free(global->player[id].name);
 	if(global->player[id].host) g_free(global->player[id].host);
 	if(global->player[id].image) g_free(global->player[id].image);
 
-	memset(&global->player[id], 0, sizeof(_player) );
+	memset(&(global->player[id]), 0, sizeof(_player) );
 }
