@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2010 Rochet Sylvain
+ *  Copyright © 2002-2014 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,12 +19,17 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifndef GLOBAL_H
+#define GLOBAL_H
+
 /* ** contain global variables */
 
 #include <gtk/gtk.h>
 #include <time.h>
 #include <sys/time.h>
 
+#include "engine.h"
+
 #ifndef WIN32
 typedef int socket_t;
 #else
@@ -35,10 +40,10 @@ typedef SOCKET socket_t;
 /* -- hierarchy
  *
  *  connection[]             > connections status
+ *  player[]                 > player game data
  *  game[]                   > game data
  *    |-> estate[]           > estate game data
  *    |-> group[]            > group game data
- *    |-> player[]           > player game data
  *    |-> card[]             > card removed from stack (needed for trade)
  *    |-> trade[]            > current trades data
  *    `-> command[]          > command game data
@@ -61,6 +66,10 @@ typedef SOCKET socket_t;
 #define MAX_PLAYERLIST_CARDS 30
 
 /* -- player data */
+#define PLAYER_ACTION_TRADE 0
+#define PLAYER_ACTION_VERSION 1
+#define PLAYER_ACTION_DATE 2
+#define PLAYER_ACTION_PING 3
 typedef struct {
 
 	guint32  playerid;
@@ -77,6 +86,7 @@ typedef struct {
 	gboolean hasturn;
 	gboolean can_roll;
 	gboolean can_buyestate;
+	gboolean spectator;
 
 	guint16 location_to;
 
@@ -87,30 +97,26 @@ typedef struct {
 	GtkWidget *playerlist_LabelMoneyPlayer;
 
 	GtkWidget *playerlist_cards_gdkrgb;
-	gboolean   playerlist_cards_redraw;
-	guint16    playerlist_cards_frame;
-	guint16    playerlist_cards_pic[MAX_PLAYERLIST_CARDS];
+	eng_frame *playerlist_cards_frame;
+	eng_obj   *playerlist_cards_pic[MAX_PLAYERLIST_CARDS];
 
 	GtkWidget *playerlist_token_gdkrgb;
-	gboolean   playerlist_token_redraw;
-	guint16    playerlist_token_frame;
-	guint16    playerlist_token_pic;
-	guint16    playerlist_token_alpha;
-
-	/* playerlistt2 */
-
+	eng_frame *playerlist_token_frame;
+	eng_obj   *playerlist_token_pic;
 
 	/* board */
 	guint32 buffer_token;
 	guint32 buffer_star;
-
-	guint16 token_pic;
-	guint16 token_alpha;
+	eng_obj *token_pic;
 
 } _player;
 
 
 /* -- estate data */
+#define ESTATE_ACTION_BUILDHOUSE 0
+#define ESTATE_ACTION_SELLHOUSE 1
+#define ESTATE_ACTION_MORTGAGE 2
+#define ESTATE_ACTION_SELL 3
 typedef struct {
 
 	gchar   *name;
@@ -134,14 +140,9 @@ typedef struct {
 
 	/* board */
 	GtkWidget *gdkrgb;
-	gboolean   redraw;
-	guint16    pic;
-
-	guint16 star_alpha;
-	guint16 star_pic;
-
-	guint16 house_alpha;
-	guint16 house_pic;
+	eng_obj   *pic;
+	eng_obj   *star_pic;
+	eng_obj   *house_pic;
 
 } _estate;
 
@@ -163,36 +164,55 @@ typedef struct {
 } _card;
 
 
-#define TRADE_CURRENT_COMPONENT_MONEY 0
-#define TRADE_CURRENT_COMPONENT_ESTATE 1
-#define TRADE_CURRENT_COMPONENT_CARD 2
-
 #define TRADE_PLAYER_PARTICIPATE 0
 #define TRADE_PLAYER_ACCEPT 1
 
-#define TRADE_SELECT_MONEY 1
-#define TRADE_SELECT_ESTATE 2
-#define TRADE_SELECT_CARD 3
+#define TRADE_ACTION_REJECT 0
+#define TRADE_ACTION_ACCEPT 1
+#define TRADE_ACTION_REMOVE 2
+
+#define TRADE_TYPE_NONE 0
+#define TRADE_TYPE_MONEY 1
+#define TRADE_TYPE_ESTATE 2
+#define TRADE_TYPE_CARD 3
+enum {
+	TRADEPROPOSALLIST_COLUMN_TYPE,
+	TRADEPROPOSALLIST_COLUMN_FROM,
+	TRADEPROPOSALLIST_COLUMN_GIVES,
+	TRADEPROPOSALLIST_COLUMN_TO,
+	TRADEPROPOSALLIST_COLUMN_WHAT,
+	TRADEPROPOSALLIST_COLUMN_TYPE_ID,
+	TRADEPROPOSALLIST_COLUMN_FROM_ID,
+	TRADEPROPOSALLIST_COLUMN_TO_ID,
+	TRADEPROPOSALLIST_COLUMN_AUX_ID,
+	TRADEPROPOSALLIST_COLUMN_NUM
+};
+enum {
+	TRADEPLAYERLIST_COLUMN_NAME,
+	TRADEPLAYERLIST_BGCOLOR_NAME,
+	TRADEPLAYERLIST_COLUMN_NUM
+};
 /* -- trade data */
 typedef struct {
 
 	gboolean open;
 
 	gint32 tradeid;
+	gint32 revision;
 	gboolean player[MAX_PLAYERS][2];
 	guint8 current_component;
 
 	GtkWidget *TradeWin;
-	GtkWidget *PlayerCList;
+	GtkWidget *PlayerList;
 	GtkWidget *FrameComponent;
-	GtkWidget *ProposalCList;
+	GtkWidget *ProposalList;
 	GtkWidget *ComponentBox;
 	GtkWidget *SubComponentBox;
 
 	guint8  select_type;  // money, estate, card
 	guint32 select_from;
 	guint32 select_to;
-	guint32 select_value;  // estateid, cardid
+	guint32 select_aux;   // estateid, cardid
 
 } _trade;
 
@@ -216,12 +236,19 @@ typedef struct {
 #define GAME_STATUS_INIT 2
 #define GAME_STATUS_RUN 3
 #define GAME_STATUS_END 4
-#define GAME_TYPE_CREATE 1
-#define GAME_TYPE_JOIN 2
-typedef struct {
+enum {
+	PLAYERLIST_COLUMN_NAME,
+	PLAYERLIST_COLUMN_HOST,
+	PLAYERLIST_COLUMN_NUM
+};
 
-	guint8 status, type;
+typedef struct game_s game;
+struct game_s {
+	game *next;
+
+	guint8 status;
 	gint32 gameid;
+	gint32 master;
 
 	_estate estate[MAX_ESTATES];
 	_group group[MAX_GROUPS];
@@ -229,14 +256,10 @@ typedef struct {
 	_trade trade[MAX_TRADES];
 	_command command[MAX_COMMANDS];
 
-	guint16 connectid;
 	time_t start_time;
-	guint16 board_frame;
+	eng_frame *board_frame;
 
-	guint32 my_playerid;
-	gboolean i_am_the_master;
-
-	GtkWidget *PlayerCList;
+	GtkWidget *PlayerList;
 
 	GtkWidget *BoxMessageAndChat;
 	gboolean   chat_float, message_float; // 0 = fixed, 1 = float
@@ -250,10 +273,10 @@ typedef struct {
 
 	guint32 timeout_token;
 
-} _game;
-
-_game *game;
+};
 
+game *currentgame;  /* Currently played game entry */
+game *game_h;  /* Game list */
 
 
 /* -- connect data */
@@ -271,6 +294,9 @@ typedef struct {
 	gchar   *host;
 	gint32  port;
 	socket_t socket;
+	GIOChannel *channel;
+	gint   event_source_id;
+
 	gchar  *server_version;
 
 	gchar *buffer_in;
@@ -327,6 +353,11 @@ typedef struct  {
 
 
 /* themes database */
+enum {
+	THEMELIST_COLUMN_NAME,
+	THEMELIST_COLUMN_ID,
+	THEMELIST_COLUMN_NUM
+};
 typedef struct {
 
 	gboolean open;
@@ -352,68 +383,58 @@ typedef struct  {
 
 	/* pngfile logo */
 	gchar  *pngfile_logo_filename;
-	gboolean pngfile_logo_need_alpha;
-	guint8  *pngfile_logo_image, *pngfile_logo_alpha;
+	guint8  *pngfile_logo_buf;
 
 	/* pngfile board */
 	gchar  *pngfile_board_filename;
-	gboolean pngfile_board_need_alpha;
 	guint16  pngfile_board_x[MAX_ESTATES],     pngfile_board_y[MAX_ESTATES];
 	guint16  pngfile_board_width[MAX_ESTATES], pngfile_board_height[MAX_ESTATES];
-	guint8  *pngfile_board_image[MAX_ESTATES];
+	guint8  *pngfile_board_buf[MAX_ESTATES];
 
 	/* pngfile_tokens */
 	gchar  *pngfile_token_filename;
-	gboolean pngfile_token_need_alpha;
 	guint16  pngfile_token_x[MAX_PLAYERS],      pngfile_token_y[MAX_PLAYERS];
 	guint16  pngfile_token_width[MAX_PLAYERS],  pngfile_token_height[MAX_PLAYERS];
-	guint8  *pngfile_token_image[MAX_PLAYERS], *pngfile_token_alpha[MAX_PLAYERS];
+	guint8  *pngfile_token_buf[MAX_PLAYERS];
 
 	/* pngfile_stars */
 	gchar  *pngfile_star_filename;
-	gboolean pngfile_star_need_alpha;
 	guint16  pngfile_star_x[MAX_PLAYERS],      pngfile_star_y[MAX_PLAYERS];
 	guint16  pngfile_star_width[MAX_PLAYERS],  pngfile_star_height[MAX_PLAYERS];
-	guint8  *pngfile_star_image[MAX_PLAYERS], *pngfile_star_alpha[MAX_PLAYERS];
+	guint8  *pngfile_star_buf[MAX_PLAYERS];
 
 	/* pngfile_stars_m (mortgaged) */
 	gchar  *pngfile_star_m_filename;
-	gboolean pngfile_star_m_need_alpha;
 	guint16  pngfile_star_m_x[MAX_PLAYERS],      pngfile_star_m_y[MAX_PLAYERS];
 	guint16  pngfile_star_m_width[MAX_PLAYERS],  pngfile_star_m_height[MAX_PLAYERS];
-	guint8  *pngfile_star_m_image[MAX_PLAYERS], *pngfile_star_m_alpha[MAX_PLAYERS];
+	guint8  *pngfile_star_m_buf[MAX_PLAYERS];
 
 	/* pngfile_cards */
 	gchar  *pngfile_card_filename;
-	gboolean pngfile_card_need_alpha;
 	guint16  pngfile_card_x[MAX_PLAYERLIST_CARDS],     pngfile_card_y[MAX_PLAYERLIST_CARDS];
 	guint16  pngfile_card_width[MAX_PLAYERLIST_CARDS], pngfile_card_height[MAX_PLAYERLIST_CARDS];
-	guint8  *pngfile_card_image[MAX_PLAYERLIST_CARDS];
+	guint8  *pngfile_card_buf[MAX_PLAYERLIST_CARDS];
 
 	/* pngfile_commands */
 	gchar  *pngfile_command_filename;
-	gboolean pngfile_command_need_alpha;
 	guint16  pngfile_command_x[MAX_COMMANDS],      pngfile_command_y[MAX_COMMANDS];
 	guint16  pngfile_command_width[MAX_COMMANDS],  pngfile_command_height[MAX_COMMANDS];
-	guint8  *pngfile_command_image[MAX_COMMANDS], *pngfile_command_alpha[MAX_COMMANDS];
+	guint8  *pngfile_command_buf[MAX_COMMANDS];
 
 	/* pngfile_horiz_houses */
 	gchar  *pngfile_horiz_house_filename;
-	gboolean pngfile_horiz_house_need_alpha;
 	guint16  pngfile_horiz_house_x[MAX_HOUSES],      pngfile_horiz_house_y[MAX_HOUSES];
 	guint16  pngfile_horiz_house_width[MAX_HOUSES],  pngfile_horiz_house_height[MAX_HOUSES];
-	guint8  *pngfile_horiz_house_image[MAX_HOUSES], *pngfile_horiz_house_alpha[MAX_HOUSES];
+	guint8  *pngfile_horiz_house_buf[MAX_HOUSES];
 
 	/* pngfile_vert_houses */
 	gchar  *pngfile_vert_house_filename;
-	gboolean pngfile_vert_house_need_alpha;
 	guint16  pngfile_vert_house_x[MAX_HOUSES],      pngfile_vert_house_y[MAX_HOUSES];
 	guint16  pngfile_vert_house_width[MAX_HOUSES],  pngfile_vert_house_height[MAX_HOUSES];
-	guint8  *pngfile_vert_house_image[MAX_HOUSES], *pngfile_vert_house_alpha[MAX_HOUSES];
+	guint8  *pngfile_vert_house_buf[MAX_HOUSES];
 
 
 	/* load flags */
-	gboolean png_intro_loaded;
 	gboolean png_game_loaded;
 
 
@@ -422,17 +443,17 @@ typedef struct  {
 
 	/* interface estate */
 	guint16 board_height, board_width;
+	guint16 board_center_x, board_center_y, board_center_width, board_center_height;
 	guint8  number_estates;
 	_interface_estate estate[MAX_ESTATES];
 
 	/* interface playerlist token */
 	guint16 playerlist_token_width, playerlist_token_height;
-	gint32  playerlist_token_bgcolor;
 
 	/* interface playerlist cards */
 	guint16 playerlist_cards_width, playerlist_cards_height;
 	guint8  playerlist_cards_alphaunowned, playerlist_cards_alphaowned, playerlist_cards_alphamortgage;
-	gint32  playerlist_cards_cardbgcolor, playerlist_cards_cardbgcolormortgage, playerlist_cards_bgcolor;
+	gint32  playerlist_cards_cardbgcolor, playerlist_cards_cardbgcolormortgage;
 	guint8  number_playerlist_card;
 	_interface_playerlist_card playerlist_card[MAX_PLAYERLIST_CARDS];
 
@@ -446,13 +467,34 @@ _data *data;
 
 
 /* -- main data */
-#define GAMECLIST_MODE_SERVERLIST 1
-#define GAMECLIST_MODE_GAMELIST   2
 #define PHASE_LOGO       1
 #define PHASE_GETGAMES   2
 #define PHASE_GAMECREATE 3
 #define PHASE_GAMEINIT   4
 #define PHASE_GAMEPLAY   5
+enum {
+	GAMELIST_COLUMN_HOST,
+	GAMELIST_COLUMN_VERSION,
+	GAMELIST_COLUMN_GAME,
+	GAMELIST_COLUMN_DESCRIPTION,
+	GAMELIST_COLUMN_PLAYERS,
+	GAMELIST_COLUMN_PORT,
+	GAMELIST_COLUMN_GAMETYPE,
+	GAMELIST_COLUMN_ID,
+	GAMELIST_COLUMN_SOURCE,
+	GAMELIST_COLUMN_NUM
+};
+enum {
+	GAMELIST_SOURCE_METASERVER,
+	GAMELIST_SOURCE_CUSTOM,
+};
+enum {
+	SERVERLIST_COLUMN_HOST,
+	SERVERLIST_COLUMN_PORT,
+	SERVERLIST_COLUMN_VERSION,
+	SERVERLIST_COLUMN_USERS,
+	SERVERLIST_COLUMN_NUM
+};
 typedef struct  {
 
 	gchar *path_home;
@@ -463,11 +505,13 @@ typedef struct  {
 	GtkWidget *MainBox;
 	GtkWidget *MainVerticalBox;
 
-	GdkPixmap *icon_pixmap;
-	GdkBitmap *icon_mask;
+	GdkPixbuf *icon_pixbuf;
 
-	GtkWidget *GameCList;
-	guint8     mode_gameclist;  // serverlist or gamelist ?
+	guint16 customserver_connectid;
+	guint16 metaserver_connectid;
+
+	GtkWidget *GameList;
+	GtkWidget *ServerList;
 	gchar     *selected_host;
 	guint32    selected_port;
 	gchar     *selected_gametype;
@@ -477,7 +521,6 @@ typedef struct  {
 	GtkWidget *HelpWin;
 
 	GtkWidget *ThemeWin;
-	gboolean   theme_redraw;
 
 	GtkWidget *Trick;  //used for reparent some widgets shouldn't be displayed
 
@@ -488,28 +531,28 @@ typedef struct  {
 
 	/* logo */
 	GtkWidget *gdkrgb_logo;
-	guint16    logo_frame, logo_pic, logo_alpha;
-	gboolean   logo_redraw;
-
-	/* gtk styles */
-	GtkStyle *style_normal;
-	GtkStyle *style_player_normal;
-	GtkStyle *style_player_turn;
+	eng_frame *logo_frame;
+	eng_obj   *logo_pic;
 
+	gboolean reconnected; /* whether we are reconnecting */
+	guint16 game_connectid;
+	gint32 my_playerid;
+	gchar   *cookie;
 } _global;
 
 _global *global;
 
 
 /* all configuration */
+#define METASERVER_HOST "meta.gradator.net"
+#define METASERVER_PORT 1240
+
 #define GAME_PLAYERLIST_POS_LEFT  0
 #define GAME_PLAYERLIST_POS_RIGHT 1
 typedef struct  {
 
 	gchar   *nickname;
 
-	gchar   *metaserver_host;
-	guint32   metaserver_port;
 	gboolean  metaserver_autogetlist;
 	gboolean  metaserver_autogetgames;
 	gboolean  metaserver_sendclientversion;
@@ -529,3 +572,4 @@ typedef struct  {
 
 _config *config;
 
+#endif /* GLOBAL_H */
