@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2010 Rochet Sylvain
+ *  Copyright © 2002-2014 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,18 +43,14 @@
 #endif
 
 /* parse a config file */
-gboolean parse_file_config(gchar *filename)  {
+gint8 parse_file_config(gchar *filename)  {
 
 	gchar *str, *ident, *tmp;
-	struct stat s;
 	FILE *file;
 
-	/* detect invalid theme */
-	if(stat(filename, &s) < 0) return FALSE;
-
 	/* parse general group */
 	file = fopen(filename, "r");
-	if (!file) return FALSE;
+	if (!file) return -1;
 
 	str = g_malloc0(512);
 
@@ -69,15 +65,7 @@ gboolean parse_file_config(gchar *filename)  {
 			if(config->nickname) g_free(config->nickname);
 			config->nickname = tmp;
 
-		} else if(! strcmp(ident, "metaserver2") )  {
-
-			tmp = parser_get_data(str, "host");
-			if(config->metaserver_host)  g_free(config->metaserver_host);
-			config->metaserver_host = tmp;
-
-			tmp = parser_get_data(str, "port");
-			config->metaserver_port = atoi( (gchar*)tmp );
-			g_free(tmp);
+		} else if(! strcmp(ident, "metaserver") )  {
 
 			tmp = parser_get_data(str, "autogetgames");
 			config->metaserver_autogetgames = atoi( (gchar*)tmp );
@@ -91,7 +79,7 @@ gboolean parse_file_config(gchar *filename)  {
 			config->metaserver_sendclientversion = atoi( (gchar*)tmp );
 			g_free(tmp);
 
-		} else if(! strcmp(ident, "anotherserver") )  {
+		} else if(! strcmp(ident, "customserver") )  {
 
 			tmp = parser_get_data(str, "host");
 			if(config->getgames_host) g_free(config->getgames_host);
@@ -139,12 +127,12 @@ gboolean parse_file_config(gchar *filename)  {
 	fclose(file);
 	g_free(str);
 
-	return TRUE;
+	return 0;
 }
 
 
 /* parse the data file interface.xml */
-gboolean xmlparse_file_interface()  {
+gint8 xmlparse_file_interface()  {
 
 	xmlDocPtr doc;
 	xmlNodePtr cur, cur2;
@@ -158,17 +146,17 @@ gboolean xmlparse_file_interface()  {
 	doc = xmlParseFile(filename);
 	g_free(filename);
 
-	if(doc == NULL)  return FALSE;
+	if(doc == NULL)  return -1;
 
 	cur = xmlDocGetRootElement(doc);
 	if (cur == NULL) {
 		xmlFreeDoc(doc);
-		return FALSE;
+		return -1;
 	}
 
 	if( xmlStrcmp(cur->name, DATAFILE_XMLROOTELEMENT) )  {
 		xmlFreeDoc(doc);
-		return FALSE;
+		return -1;
 	}
 
 	cur = cur->xmlChildrenNode;
@@ -182,10 +170,6 @@ gboolean xmlparse_file_interface()  {
 
 			data->pngfile_logo_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
 
-			tmp = xmlGetProp(cur, (xmlChar*)"need_alpha");
-			data->pngfile_logo_need_alpha = atoi( (gchar*)tmp );
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 			do {
 
@@ -205,10 +189,6 @@ gboolean xmlparse_file_interface()  {
 
 			data->pngfile_board_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
 
-			tmp = xmlGetProp(cur, (xmlChar*)"need_alpha");
-			data->pngfile_board_need_alpha = atoi( (gchar*)tmp );
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 			do {
 
@@ -244,10 +224,6 @@ gboolean xmlparse_file_interface()  {
 
 			data->pngfile_token_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
 
-			tmp = xmlGetProp(cur, (xmlChar*)"need_alpha");
-			data->pngfile_token_need_alpha = atoi( (gchar*)tmp );
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 			do {
 
@@ -283,10 +259,6 @@ gboolean xmlparse_file_interface()  {
 
 			data->pngfile_card_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
 
-			tmp = xmlGetProp(cur, (xmlChar*)"need_alpha");
-			data->pngfile_card_need_alpha = atoi( (gchar*)tmp );
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 			do {
 
@@ -322,10 +294,6 @@ gboolean xmlparse_file_interface()  {
 
 			data->pngfile_star_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
 
-			tmp = xmlGetProp(cur, (xmlChar*)"need_alpha");
-			data->pngfile_star_need_alpha = atoi( (gchar*)tmp );
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 			do {
 
@@ -361,10 +329,6 @@ gboolean xmlparse_file_interface()  {
 
 			data->pngfile_star_m_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
 
-			tmp = xmlGetProp(cur, (xmlChar*)"need_alpha");
-			data->pngfile_star_m_need_alpha = atoi( (gchar*)tmp );
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 			do {
 
@@ -400,10 +364,6 @@ gboolean xmlparse_file_interface()  {
 
 			data->pngfile_command_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
 
-			tmp = xmlGetProp(cur, (xmlChar*)"need_alpha");
-			data->pngfile_command_need_alpha = atoi( (gchar*)tmp );
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 			do {
 
@@ -439,10 +399,6 @@ gboolean xmlparse_file_interface()  {
 
 			data->pngfile_horiz_house_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
 
-			tmp = xmlGetProp(cur, (xmlChar*)"need_alpha");
-			data->pngfile_horiz_house_need_alpha = atoi( (gchar*)tmp );
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 			do {
 
@@ -478,10 +434,6 @@ gboolean xmlparse_file_interface()  {
 
 			data->pngfile_vert_house_filename = (gchar*)xmlGetProp(cur, (xmlChar*)"filename");
 
-			tmp = xmlGetProp(cur, (xmlChar*)"need_alpha");
-			data->pngfile_vert_house_need_alpha = atoi( (gchar*)tmp );
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 			do {
 
@@ -529,6 +481,22 @@ gboolean xmlparse_file_interface()  {
 			data->board_height = atoi( (gchar*)tmp );
 			g_free(tmp);
 
+			tmp = xmlGetProp(cur, (xmlChar*)"center_x");
+			data->board_center_x = atoi( (gchar*)tmp );
+			g_free(tmp);
+
+			tmp = xmlGetProp(cur, (xmlChar*)"center_y");
+			data->board_center_y = atoi( (gchar*)tmp );
+			g_free(tmp);
+
+			tmp = xmlGetProp(cur, (xmlChar*)"center_width");
+			data->board_center_width = atoi( (gchar*)tmp );
+			g_free(tmp);
+
+			tmp = xmlGetProp(cur, (xmlChar*)"center_height");
+			data->board_center_height = atoi( (gchar*)tmp );
+			g_free(tmp);
+
 			cur2 = cur->xmlChildrenNode;
 
 			do {
@@ -625,15 +593,6 @@ gboolean xmlparse_file_interface()  {
 			tmp = xmlGetProp(cur, (xmlChar*)"height");
 			data->playerlist_token_height = atoi( (gchar*)tmp );
 			g_free(tmp);
-
-			tmp = xmlGetProp(cur, (xmlChar*)"bgcolor");
-			if( atoi( (gchar*)tmp ) < 0 )
-				data->playerlist_token_bgcolor = -1;
-			else  {
-				sscanf((gchar*)tmp, "0x%6X", &color);
-				data->playerlist_token_bgcolor = color;
-			}
-			g_free(tmp);
 		}
 
 		/* parse player cards interface data */
@@ -694,15 +653,6 @@ gboolean xmlparse_file_interface()  {
 			data->playerlist_cards_cardbgcolormortgage = color;
 			g_free(tmp);
 
-			tmp = xmlGetProp(cur, (xmlChar*)"bgcolor");
-			if( atoi((gchar*)tmp) < 0 )
-				data->playerlist_cards_bgcolor = -1;
-			else  {
-				sscanf((gchar*)tmp, "0x%6X", &color);
-				data->playerlist_cards_bgcolor = color;
-			}
-			g_free(tmp);
-
 			cur2 = cur->xmlChildrenNode;
 
 			do {
@@ -745,374 +695,180 @@ gboolean xmlparse_file_interface()  {
 	} while(cur != NULL);
 
 	xmlFreeDoc(doc);
-	return TRUE;
+	return 0;
 }
 
 
 /* read all intro pngs images */
-gboolean intro_load_pngs()   {
-
-	png_structp png_ptr;
-	png_infop info_ptr;
+gint8 intro_load_pngs()   {
 	_png_imagedata png_imagedata;
 	gchar *filename;
-	FILE *infile;
+	gint8 ret = 0;
 
-	if(data->png_intro_loaded)  return TRUE;
+	memset(&png_imagedata, 0, sizeof(_png_imagedata));
 
 	/* ---- load logo file */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_logo_filename, NULL);
-
-	if (! (infile = fopen(filename, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", filename);
-		g_free(filename);
-		return FALSE;
+	if (readpng_read(filename, &png_imagedata)) {
+		ret = -1;
+		goto free_and_return;
 	}
+
+	data->logo_width = readpng_get_width(&png_imagedata);
+	data->logo_height = readpng_get_height(&png_imagedata);
+	data->pngfile_logo_buf = readpng_get_rgba(&png_imagedata);
+
+free_and_return:
 	g_free(filename);
-
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
-		fclose(infile);
-		return FALSE;
-	}
-
-	if(data->pngfile_logo_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
-	data->logo_width = png_imagedata.width;
-	data->logo_height = png_imagedata.height;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
-		return FALSE;
-	}
-	fclose(infile);
-
-	data->pngfile_logo_image = readpng_get_image(&png_imagedata);
-	data->pngfile_logo_alpha = readpng_get_alpha(&png_imagedata);
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
-
-	data->png_intro_loaded = TRUE;
-	return TRUE;
+	readpng_cleanup(&png_imagedata);
+	return ret;
 }
 
 
 /* free all intro pngs images */
 void intro_free_pngs()  {
 
-	if(! data->png_intro_loaded)  return;
-
-	if(data->pngfile_logo_image) g_free(data->pngfile_logo_image);
-	if(data->pngfile_logo_alpha) g_free(data->pngfile_logo_alpha);
-	data->pngfile_logo_image = 0;
-	data->pngfile_logo_alpha = 0;
-
-	data->png_intro_loaded = FALSE;
+	if (data->pngfile_logo_buf) {
+		g_free(data->pngfile_logo_buf);
+		data->pngfile_logo_buf = NULL;
+	}
 }
 
 
 /* read all game pngs images */
-gboolean game_load_pngs()   {
-
-	png_structp png_ptr;
-	png_infop info_ptr;
+gint8 game_load_pngs()   {
 	_png_imagedata png_imagedata;
 	gchar *filename;
-	FILE *infile;
 	guint32 i;
+	gint8 ret = 0;
 
-	if(data->png_game_loaded)  return TRUE;
+	if (data->png_game_loaded) {
+		return 0;
+	}
 
 	/* ---- load board file */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_board_filename, NULL);
-
-	if (! (infile = fopen(filename, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", filename);
-		g_free(filename);
-		return FALSE;
+	if (readpng_read(filename, &png_imagedata)) {
+		ret = -1;
+		goto free_and_return;
 	}
+
+	for (i = 0 ; data->pngfile_board_width[i] ; i++) {
+		data->pngfile_board_buf[i] = readpng_get_rgba_crop(&png_imagedata, data->pngfile_board_x[i], data->pngfile_board_y[i], data->pngfile_board_width[i], data->pngfile_board_height[i]);
+	}
+
 	g_free(filename);
-
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
-		fclose(infile);
-		return FALSE;
-	}
-
-	if(data->pngfile_board_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, FALSE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
-		return FALSE;
-	}
-	fclose(infile);
-
-	for(i = 0 ; data->pngfile_board_width[i] ; i++)  {
-
-		data->pngfile_board_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_board_x[i], data->pngfile_board_y[i], data->pngfile_board_width[i], data->pngfile_board_height[i]);
-	}
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
+	readpng_cleanup(&png_imagedata);
 
 
 	/* ---- load token file */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_token_filename, NULL);
-
-	if (! (infile = fopen(filename, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", filename);
-		g_free(filename);
-		return FALSE;
+	if (readpng_read(filename, &png_imagedata)) {
+		ret = -1;
+		goto free_and_return;
 	}
+
+	for (i = 0 ; data->pngfile_token_width[i] ; i++) {
+		data->pngfile_token_buf[i] = readpng_get_rgba_crop(&png_imagedata, data->pngfile_token_x[i], data->pngfile_token_y[i], data->pngfile_token_width[i], data->pngfile_token_height[i]);
+	}
+
 	g_free(filename);
-
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
-		fclose(infile);
-		return FALSE;
-	}
-
-	if(data->pngfile_token_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
-		return FALSE;
-	}
-	fclose(infile);
-
-	for(i = 0 ; data->pngfile_token_width[i] ; i++)  {
-
-		data->pngfile_token_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_token_x[i], data->pngfile_token_y[i], data->pngfile_token_width[i], data->pngfile_token_height[i]);
-		data->pngfile_token_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_token_x[i], data->pngfile_token_y[i], data->pngfile_token_width[i], data->pngfile_token_height[i]);
-	}
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
+	readpng_cleanup(&png_imagedata);
 
 
 	/* ---- load star file */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_star_filename, NULL);
-
-	if (! (infile = fopen(filename, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", filename);
-		g_free(filename);
-		return FALSE;
+	if (readpng_read(filename, &png_imagedata)) {
+		ret = -1;
+		goto free_and_return;
 	}
+
+	for (i = 0 ; data->pngfile_star_width[i] ; i++) {
+		data->pngfile_star_buf[i] = readpng_get_rgba_crop(&png_imagedata, data->pngfile_star_x[i], data->pngfile_star_y[i], data->pngfile_star_width[i], data->pngfile_star_height[i]);
+	}
+
 	g_free(filename);
-
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
-		fclose(infile);
-		return FALSE;
-	}
-
-	if(data->pngfile_star_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
-		return FALSE;
-	}
-	fclose(infile);
-
-	for(i = 0 ; data->pngfile_star_width[i] ; i++)  {
-
-		data->pngfile_star_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_star_x[i], data->pngfile_star_y[i], data->pngfile_star_width[i], data->pngfile_star_height[i]);
-		data->pngfile_star_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_star_x[i], data->pngfile_star_y[i], data->pngfile_star_width[i], data->pngfile_star_height[i]);
-	}
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
+	readpng_cleanup(&png_imagedata);
 
 
 	/* ---- load star_m file (mortgaged) */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_star_m_filename, NULL);
-
-	if (! (infile = fopen(filename, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", filename);
-		g_free(filename);
-		return FALSE;
+	if (readpng_read(filename, &png_imagedata)) {
+		ret = -1;
+		goto free_and_return;
 	}
+
+	for (i = 0 ; data->pngfile_star_m_width[i] ; i++) {
+		data->pngfile_star_m_buf[i] = readpng_get_rgba_crop(&png_imagedata, data->pngfile_star_m_x[i], data->pngfile_star_m_y[i], data->pngfile_star_m_width[i], data->pngfile_star_m_height[i]);
+	}
+
 	g_free(filename);
-
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
-		fclose(infile);
-		return FALSE;
-	}
-
-	if(data->pngfile_star_m_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
-		return FALSE;
-	}
-	fclose(infile);
-
-	for(i = 0 ; data->pngfile_star_m_width[i] ; i++)  {
-
-		data->pngfile_star_m_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_star_m_x[i], data->pngfile_star_m_y[i], data->pngfile_star_m_width[i], data->pngfile_star_m_height[i]);
-		data->pngfile_star_m_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_star_m_x[i], data->pngfile_star_m_y[i], data->pngfile_star_m_width[i], data->pngfile_star_m_height[i]);
-	}
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
+	readpng_cleanup(&png_imagedata);
 
 
 	/* ---- load cards file */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_card_filename, NULL);
-
-	if (! (infile = fopen(filename, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", filename);
-		g_free(filename);
-		return FALSE;
+	if (readpng_read(filename, &png_imagedata)) {
+		ret = -1;
+		goto free_and_return;
 	}
+
+	for (i = 0 ; data->pngfile_card_width[i] ; i++) {
+
+		data->pngfile_card_buf[i] = readpng_get_rgba_crop(&png_imagedata, data->pngfile_card_x[i], data->pngfile_card_y[i], data->pngfile_card_width[i], data->pngfile_card_height[i]);
+	}
+
 	g_free(filename);
-
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
-		fclose(infile);
-		return FALSE;
-	}
-
-	if(data->pngfile_card_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, FALSE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
-		return FALSE;
-	}
-	fclose(infile);
-
-	for(i = 0 ; data->pngfile_card_width[i] ; i++)  {
-
-		data->pngfile_card_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_card_x[i], data->pngfile_card_y[i], data->pngfile_card_width[i], data->pngfile_card_height[i]);
-	}
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
+	readpng_cleanup(&png_imagedata);
 
 
 	/* ---- load command file */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_command_filename, NULL);
-
-	if (! (infile = fopen(filename, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", filename);
-		g_free(filename);
-		return FALSE;
+	if (readpng_read(filename, &png_imagedata)) {
+		ret = -1;
+		goto free_and_return;
 	}
+
+	for (i = 0 ; data->pngfile_command_width[i] ; i++) {
+		data->pngfile_command_buf[i] = readpng_get_rgba_crop(&png_imagedata, data->pngfile_command_x[i], data->pngfile_command_y[i], data->pngfile_command_width[i], data->pngfile_command_height[i]);
+	}
+
 	g_free(filename);
-
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
-		fclose(infile);
-		return FALSE;
-	}
-
-	if(data->pngfile_command_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
-		return FALSE;
-	}
-	fclose(infile);
-
-	for(i = 0 ; data->pngfile_command_width[i] ; i++)  {
-
-		data->pngfile_command_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_command_x[i], data->pngfile_command_y[i], data->pngfile_command_width[i], data->pngfile_command_height[i]);
-		data->pngfile_command_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_command_x[i], data->pngfile_command_y[i], data->pngfile_command_width[i], data->pngfile_command_height[i]);
-	}
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
+	readpng_cleanup(&png_imagedata);
 
 
 	/* ---- load horiz_houses file */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_horiz_house_filename, NULL);
-
-	if (! (infile = fopen(filename, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", filename);
-		g_free(filename);
-		return FALSE;
+	if (readpng_read(filename, &png_imagedata)) {
+		ret = -1;
+		goto free_and_return;
 	}
+
+	for (i = 0 ; data->pngfile_horiz_house_width[i] ; i++) {
+		data->pngfile_horiz_house_buf[i] = readpng_get_rgba_crop(&png_imagedata, data->pngfile_horiz_house_x[i], data->pngfile_horiz_house_y[i], data->pngfile_horiz_house_width[i], data->pngfile_horiz_house_height[i]);
+	}
+
 	g_free(filename);
-
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
-		fclose(infile);
-		return FALSE;
-	}
-
-	if(data->pngfile_horiz_house_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
-		return FALSE;
-	}
-	fclose(infile);
-
-	for(i = 0 ; data->pngfile_horiz_house_width[i] ; i++)  {
-
-		data->pngfile_horiz_house_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_horiz_house_x[i], data->pngfile_horiz_house_y[i], data->pngfile_horiz_house_width[i], data->pngfile_horiz_house_height[i]);
-		data->pngfile_horiz_house_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_horiz_house_x[i], data->pngfile_horiz_house_y[i], data->pngfile_horiz_house_width[i], data->pngfile_horiz_house_height[i]);
-	}
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
-
+	readpng_cleanup(&png_imagedata);
 
 
 	/* ---- load vert_houses file */
 	filename = g_strconcat(PACKAGE_DATA_DIR, "/", data->pngfile_vert_house_filename, NULL);
-
-	if (! (infile = fopen(filename, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", filename);
-		g_free(filename);
-		return FALSE;
-	}
-	g_free(filename);
-
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", filename);
-		fclose(infile);
-		return FALSE;
+	if (readpng_read(filename, &png_imagedata)) {
+		ret = -1;
+		goto free_and_return;
 	}
 
-	if(data->pngfile_vert_house_need_alpha  &&  !png_imagedata.have_alpha) return FALSE;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", filename);
-		return FALSE;
+	for (i = 0 ; data->pngfile_vert_house_width[i] ; i++) {
+		data->pngfile_vert_house_buf[i] = readpng_get_rgba_crop(&png_imagedata, data->pngfile_vert_house_x[i], data->pngfile_vert_house_y[i], data->pngfile_vert_house_width[i], data->pngfile_vert_house_height[i]);
 	}
-	fclose(infile);
-
-	for(i = 0 ; data->pngfile_vert_house_width[i] ; i++)  {
-
-		data->pngfile_vert_house_image[i] = readpng_get_image_zone(&png_imagedata, data->pngfile_vert_house_x[i], data->pngfile_vert_house_y[i], data->pngfile_vert_house_width[i], data->pngfile_vert_house_height[i]);
-		data->pngfile_vert_house_alpha[i] = readpng_get_alpha_zone(&png_imagedata, data->pngfile_vert_house_x[i], data->pngfile_vert_house_y[i], data->pngfile_vert_house_width[i], data->pngfile_vert_house_height[i]);
-	}
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
 
 	data->png_game_loaded = TRUE;
-	return TRUE;
+
+free_and_return:
+	readpng_cleanup(&png_imagedata);
+	if(filename) g_free(filename);
+	return ret;
 }
 
 
@@ -1124,126 +880,60 @@ void game_free_pngs()  {
 	if(! data->png_game_loaded)  return;
 
 	for(i = 0 ; data->pngfile_board_width[i] ; i++)  {
-
-		if(data->pngfile_board_image[i])  {
-			g_free(data->pngfile_board_image[i]);
-			data->pngfile_board_image[i] = 0;
+		if(data->pngfile_board_buf[i])  {
+			g_free(data->pngfile_board_buf[i]);
+			data->pngfile_board_buf[i] = NULL;
 		}
 	}
 
 	for(i = 0 ; data->pngfile_token_width[i] ; i++)  {
-
-		if(data->pngfile_token_image[i])  {
-			g_free(data->pngfile_token_image[i]);
-			data->pngfile_token_image[i] = 0;
-		}
-		if(data->pngfile_token_alpha[i])  {
-			g_free(data->pngfile_token_alpha[i]);
-			data->pngfile_token_alpha[i] = 0;
+		if(data->pngfile_token_buf[i])  {
+			g_free(data->pngfile_token_buf[i]);
+			data->pngfile_token_buf[i] = NULL;
 		}
 	}
 
-
 	for(i = 0 ; data->pngfile_star_width[i] ; i++)  {
-
-		if(data->pngfile_star_image[i])  {
-			g_free(data->pngfile_star_image[i]);
-			data->pngfile_star_image[i] = 0;
-		}
-		if(data->pngfile_star_alpha[i])  {
-			g_free(data->pngfile_star_alpha[i]);
-			data->pngfile_star_alpha[i] = 0;
+		if(data->pngfile_star_buf[i])  {
+			g_free(data->pngfile_star_buf[i]);
+			data->pngfile_star_buf[i] = NULL;
 		}
 	}
 
 	for(i = 0 ; data->pngfile_star_m_width[i] ; i++)  {
-
-		if(data->pngfile_star_m_image[i])  {
-			g_free(data->pngfile_star_m_image[i]);
-			data->pngfile_star_m_image[i] = 0;
-		}
-		if(data->pngfile_star_m_alpha[i])  {
-			g_free(data->pngfile_star_m_alpha[i]);
-			data->pngfile_star_m_alpha[i] = 0;
+		if(data->pngfile_star_m_buf[i])  {
+			g_free(data->pngfile_star_m_buf[i]);
+			data->pngfile_star_m_buf[i] = NULL;
 		}
 	}
 
 	for(i = 0 ; data->pngfile_card_width[i] ; i++)  {
-
-		if(data->pngfile_card_image[i])  {
-			g_free(data->pngfile_card_image[i]);
-			data->pngfile_card_image[i] = 0;
+		if(data->pngfile_card_buf[i])  {
+			g_free(data->pngfile_card_buf[i]);
+			data->pngfile_card_buf[i] = NULL;
 		}
 	}
 
 	for(i = 0 ; data->pngfile_command_width[i] ; i++)  {
-
-		if(data->pngfile_command_image[i])  {
-			g_free(data->pngfile_command_image[i]);
-			data->pngfile_command_image[i] = 0;
-		}
-		if(data->pngfile_command_alpha[i])  {
-			g_free(data->pngfile_command_alpha[i]);
-			data->pngfile_command_alpha[i] = 0;
+		if(data->pngfile_command_buf[i])  {
+			g_free(data->pngfile_command_buf[i]);
+			data->pngfile_command_buf[i] = NULL;
 		}
 	}
 
 	for(i = 0 ; data->pngfile_horiz_house_width[i] ; i++)  {
-
-		if(data->pngfile_horiz_house_image[i])  {
-			g_free(data->pngfile_horiz_house_image[i]);
-			data->pngfile_horiz_house_image[i] = 0;
-		}
-		if(data->pngfile_horiz_house_alpha[i])  {
-			g_free(data->pngfile_horiz_house_alpha[i]);
-			data->pngfile_horiz_house_alpha[i] = 0;
+		if(data->pngfile_horiz_house_buf[i])  {
+			g_free(data->pngfile_horiz_house_buf[i]);
+			data->pngfile_horiz_house_buf[i] = NULL;
 		}
 	}
 
 	for(i = 0 ; data->pngfile_vert_house_width[i] ; i++)  {
-
-		if(data->pngfile_vert_house_image[i])  {
-			g_free(data->pngfile_vert_house_image[i]);
-			data->pngfile_vert_house_image[i] = 0;
-		}
-		if(data->pngfile_vert_house_alpha[i])  {
-			g_free(data->pngfile_vert_house_alpha[i]);
-			data->pngfile_vert_house_alpha[i] = 0;
+		if(data->pngfile_vert_house_buf[i])  {
+			g_free(data->pngfile_vert_house_buf[i]);
+			data->pngfile_vert_house_buf[i] = NULL;
 		}
 	}
 
 	data->png_game_loaded = FALSE;
 }
-
-
-void init_prototype_gtk_style()  {
-
-	GtkWidget *label;
-	GtkStyle  *rcstyle;
-
-	/* create a dummy label so we can fetch the associated rc style */
-	label = gtk_label_new ("");
-	rcstyle = gtk_rc_get_style (label);
-	gtk_widget_destroy (label);
-
-	/* this is needed for some (broken?) gtk themes */
-	if (rcstyle == NULL)
-		rcstyle = gtk_style_new ();
-	else
-		gtk_style_ref (rcstyle);
-
-
-	global->style_normal = gtk_style_copy (rcstyle);
-
-	global->style_player_normal = gtk_style_copy (rcstyle);
-	global->style_player_normal->fg[0].red   = 0x0000;
-	global->style_player_normal->fg[0].green = 0x0000;
-	global->style_player_normal->fg[0].blue  = 0x0000;
-
-	global->style_player_turn = gtk_style_copy (rcstyle);
-	global->style_player_turn->fg[0].red   = 0xffff;
-	global->style_player_turn->fg[0].green = 0x0000;
-	global->style_player_turn->fg[0].blue  = 0x0000;
-
-	gtk_style_unref (rcstyle);
-}
