@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2010 Rochet Sylvain
+ *  Copyright © 2002-2014 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,10 +24,9 @@
 #define DATAFILE_XMLROOTELEMENT (xmlChar*)"gtkatlantic"
 
 
-gboolean parse_file_config(gchar *filename);
-gboolean xmlparse_file_interface();
-gboolean intro_load_pngs();
+gint8 parse_file_config(gchar *filename);
+gint8 xmlparse_file_interface();
+gint8 intro_load_pngs();
 void intro_free_pngs();
-gboolean game_load_pngs();
+gint8 game_load_pngs();
 void game_free_pngs();
-void init_prototype_gtk_style();
