@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2010 Rochet Sylvain
+ *  Copyright © 2002-2014 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,13 +24,12 @@
 #include <unistd.h>
 
 #include <gtk/gtk.h>
-#include <gdk/gdkrgb.h>
+#include <gdk/gdk.h>
 
 #include "config.h"
 
 #include "global.h"
 #include "interface.h"
-#include "main.h"
 #include "load.h"
 #include "client.h"
 #include "game.h"
@@ -58,58 +57,23 @@ int main(int argc, char **argv)  {
 	read_config_files();
 
 	gtk_init(&argc, &argv);
-	gdk_rgb_init();
 
-	engine_init();
-	engine_minimize_memory_usage();
+	eng_init();
 
-	if(! xmlparse_file_interface() ) exit(-1);
-	if(! intro_load_pngs() ) exit(-1);
-	init_prototype_gtk_style();
+	if( xmlparse_file_interface() ) exit(-1);
+	if( intro_load_pngs() ) exit(-1);
 
 	interface_create_mainwindow();
 	interface_create_menubar();
 	interface_create_logo();
 
-	/* ---- add timeout */
-
-	/* I think gtk use gettimeofday() or a methode like this to determine   *
-	 * elapsed time, this is quite imprecise. In result if you set timeout  *
-	 * to 40 ms you have only 20 hertz and not 25 ...                       */
-	gtk_timeout_add(90, client_fonct, NULL);     // 10 hertz
-	gtk_timeout_add(40, update_display, NULL);   // 20 hertz
-
-
 	gtk_main();
-	return FALSE;
+
+	/* cleanup */
+	intro_free_pngs();
+	game_free_pngs();
+#ifdef WIN32
+	WSACleanup();
+#endif
+	return 0;
 }
-
-
-
-#include <sys/time.h>
-int timea, timeb, elp, old, fps;
-struct timeval tv;
-
-gboolean test_frequency() {
-
-	gettimeofday(&tv, NULL);
-	timeb = tv.tv_usec;
-	elp = timeb - timea;
-	if(elp < 0) elp = 1000000 - timea + timeb;
-
-//	printf("time elapsed = %d\n", elp);
-	timea = tv.tv_usec;
-
-	if(old != tv.tv_sec)  {
-
-		printf("====> fps = %d\n", fps);
-		fps = 0;
-		old = tv.tv_sec;
-	}
-	fps++;
-
-	return(TRUE);
-}
-
-
-
