@@ -2,7 +2,7 @@
  *     gtkatlantic - the gtk+ monopd client, enjoy network monopoly games
  *
  *
- *  Copyright (C) 2002-2010 Rochet Sylvain
+ *  Copyright © 2002-2014 Sylvain Rochet
  *
  *  gtkatlantic is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -228,140 +228,144 @@ void theme_build_selection_win()  {
 					GtkWidget *HBoxButtons;
 						GtkWidget *Button;
 				GtkWidget *VBoxInfo;
-					GtkWidget *TableInfo;
+					GtkWidget *GridInfo;
 						GtkWidget *Alignment;
 							GtkWidget *Label;
 				GtkWidget *PreviewFrame;
 
 	GtkWidget *tmp;
 	gchar *str;
+	GtkTreeSelection *select;
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+	GtkListStore *store;
 
 	if(global->ThemeWin)  return;
 
 	global->ThemeWin = ThemeWin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title(GTK_WINDOW(ThemeWin), "GtkAtlantic: Theme Selection");
-	gtk_window_set_policy(GTK_WINDOW(ThemeWin), FALSE, TRUE, TRUE);
-	gtk_widget_realize(ThemeWin);
-	gdk_window_set_icon(ThemeWin->window, NULL, global->icon_pixmap, global->icon_mask);
-	gtk_signal_connect(GTK_OBJECT(ThemeWin), "delete_event", (GtkSignalFunc)CallBack_ThemeWin_Delete, NULL);
-	gtk_signal_connect(GTK_OBJECT(ThemeWin), "destroy", (GtkSignalFunc)CallBack_ThemeWin_Destroyed, NULL);
+	gtk_window_set_icon(GTK_WINDOW(global->MainWin), global->icon_pixbuf);
+	g_signal_connect(G_OBJECT(ThemeWin), "delete_event", G_CALLBACK(CallBack_ThemeWin_Delete), NULL);
+	g_signal_connect(G_OBJECT(ThemeWin), "destroy", G_CALLBACK(CallBack_ThemeWin_Destroyed), NULL);
 
-	VBox = gtk_vbox_new(FALSE, 0);
+	VBox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(VBox), 5);
 	gtk_container_add(GTK_CONTAINER(ThemeWin), VBox);
 
-	HBox = gtk_hbox_new(FALSE, 5);
+	HBox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
 	gtk_box_pack_start(GTK_BOX(VBox), HBox, TRUE, TRUE, 0);
 
-	VBoxLeft = gtk_vbox_new(FALSE, 5);
+	VBoxLeft = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_box_pack_start(GTK_BOX(HBox), VBoxLeft, TRUE, TRUE, 0);
 
 	/* theme list */
 	ScrollThemeList = gtk_scrolled_window_new(NULL, NULL);
-	gtk_widget_set_usize(ScrollThemeList, 140, 0);
+	gtk_widget_set_size_request(ScrollThemeList, 140, -1);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ScrollThemeList), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start(GTK_BOX(VBoxLeft), ScrollThemeList, TRUE, TRUE, 0);
 
-	ThemeCList = gtk_clist_new(2);
-	gtk_clist_set_column_auto_resize(GTK_CLIST(ThemeCList), 0, TRUE);
-	gtk_clist_set_column_auto_resize(GTK_CLIST(ThemeCList), 1, TRUE);
-	gtk_clist_set_column_visibility(GTK_CLIST(ThemeCList), 1, FALSE);
-	gtk_clist_column_titles_passive(GTK_CLIST(ThemeCList));
-	gtk_clist_set_reorderable(GTK_CLIST(ThemeCList), TRUE);
-	gtk_clist_set_selection_mode(GTK_CLIST(ThemeCList), GTK_SELECTION_SINGLE);
-	gtk_object_set_data(GTK_OBJECT(ThemeWin), "theme_list", ThemeCList);
+	ThemeCList = gtk_tree_view_new();
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_list", ThemeCList);
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(ThemeCList), FALSE);
 	gtk_container_add(GTK_CONTAINER(ScrollThemeList), ThemeCList);
+	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(ThemeCList));
+	gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
+	g_signal_connect(G_OBJECT(select), "changed", G_CALLBACK(Callback_ThemeList_Select), NULL);
 
-	gtk_signal_connect(GTK_OBJECT(ThemeCList), "unselect_row", (GtkSignalFunc)Callback_ThemeList_Unselect, NULL);
-	gtk_signal_connect_after(GTK_OBJECT(ThemeCList), "select_row", (GtkSignalFunc)Callback_ThemeList_Select, NULL);
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes("Theme", renderer, "text", THEMELIST_COLUMN_NAME, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(ThemeCList), column);
+
+	store = gtk_list_store_new(TRADEPLAYERLIST_COLUMN_NUM, G_TYPE_STRING, G_TYPE_INT);
+	gtk_tree_view_set_model(GTK_TREE_VIEW(ThemeCList), GTK_TREE_MODEL(store));
+	g_object_unref(store);
 
 	/* hbox for buttons */
-	HBoxButtons = gtk_hbox_new(FALSE, 5);
+	HBoxButtons = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 5);
 	gtk_box_pack_start(GTK_BOX(VBoxLeft), HBoxButtons, FALSE, FALSE, 0);
 
 	Button = gtk_button_new_with_label("Apply");
-	gtk_signal_connect(GTK_OBJECT(Button), "clicked", (GtkSignalFunc)CallBack_Theme_Apply, NULL);
+	g_signal_connect(G_OBJECT(Button), "clicked", G_CALLBACK(CallBack_Theme_Apply), NULL);
 	gtk_box_pack_start(GTK_BOX(HBoxButtons), Button, TRUE, TRUE, 0);
 
 	Button = gtk_button_new_with_label("Cancel");
-	gtk_signal_connect(GTK_OBJECT(Button), "clicked", (GtkSignalFunc)CallBack_ThemeWin_Delete, NULL);
+	g_signal_connect(G_OBJECT(Button), "clicked", G_CALLBACK(CallBack_ThemeWin_Delete), NULL);
 	gtk_box_pack_start(GTK_BOX(HBoxButtons), Button, TRUE, TRUE, 0);
 
 	/* vbox for themes */
-	VBoxInfo = gtk_vbox_new(FALSE, 5);
+	VBoxInfo = gtk_box_new(GTK_ORIENTATION_VERTICAL, 5);
 	gtk_box_pack_start(GTK_BOX(HBox), VBoxInfo, TRUE, TRUE, 0);
 
 	/* table which contain theme infos */
-	TableInfo = gtk_table_new (6, 2, FALSE);
-	gtk_table_set_col_spacings(GTK_TABLE(TableInfo), 5);
-	gtk_table_set_row_spacings(GTK_TABLE(TableInfo), 2);
-	gtk_box_pack_start(GTK_BOX(VBoxInfo), TableInfo, FALSE, FALSE, 0);
+	GridInfo = gtk_grid_new();
+	gtk_grid_set_column_spacing(GTK_GRID(GridInfo), 5);
+	gtk_grid_set_row_spacing(GTK_GRID(GridInfo), 2);
+	gtk_box_pack_start(GTK_BOX(VBoxInfo), GridInfo, FALSE, FALSE, 0);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-//	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 0, 1, 0, 1, GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 0, 1, 0, 1, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 0, 1, 1);
 	Label = gtk_label_new("Name:");
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 1, 2, 0, 1, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 0, 1, 1);
 	Label = gtk_label_new("");
-	gtk_object_set_data(GTK_OBJECT(ThemeWin), "theme_name", Label);
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_name", Label);
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 0, 1, 1, 2, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 1, 1, 1);
 	Label = gtk_label_new("Type:");
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 1, 2, 1, 2, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 1, 1, 1);
 	Label = gtk_label_new("");
-	gtk_object_set_data(GTK_OBJECT(ThemeWin), "theme_type", Label);
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_type", Label);
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 0, 1, 2, 3, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 2, 1, 1);
 	Label = gtk_label_new("Version:");
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 1, 2, 2, 3, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 2, 1, 1);
 	Label = gtk_label_new("");
-	gtk_object_set_data(GTK_OBJECT(ThemeWin), "theme_version", Label);
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_version", Label);
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 0, 1, 3, 4, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 3, 1, 1);
 	Label = gtk_label_new("Author:");
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 1, 2, 3, 4, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 3, 1, 1);
 	Label = gtk_label_new("");
-	gtk_object_set_data(GTK_OBJECT(ThemeWin), "theme_author", Label);
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_author", Label);
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 0, 1, 4, 5, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 4, 1, 1);
 	Label = gtk_label_new("Email:");
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 1, 2, 4, 5, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 4, 1, 1);
 	Label = gtk_label_new("");
-	gtk_object_set_data(GTK_OBJECT(ThemeWin), "theme_email", Label);
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_email", Label);
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 0, 1, 5, 6, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 0, 5, 1, 1);
 	Label = gtk_label_new("Location:");
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	Alignment = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
-	gtk_table_attach(GTK_TABLE(TableInfo), Alignment, 1, 2, 5, 6, GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 0, 0);
+	gtk_grid_attach(GTK_GRID(GridInfo), Alignment, 1, 5, 1, 1);
 	Label = gtk_label_new("");
-	gtk_object_set_data(GTK_OBJECT(ThemeWin), "theme_location", Label);
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_location", Label);
 	gtk_container_add(GTK_CONTAINER(Alignment), Label);
 
 	/* preview */
@@ -369,17 +373,17 @@ void theme_build_selection_win()  {
 	gtk_box_pack_start(GTK_BOX(VBoxInfo), PreviewFrame, TRUE, TRUE, 0);
 
 	Alignment = gtk_alignment_new(0.5, 0.5, 0.0, 0.0);
-	gtk_object_set_data(GTK_OBJECT(ThemeWin), "theme_preview_container", Alignment);
-	gtk_widget_set_usize(Alignment, 400, 245);
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_preview_container", Alignment);
+	gtk_widget_set_size_request(Alignment, 400, 245);
 	gtk_container_add(GTK_CONTAINER(PreviewFrame), Alignment);
 
 	tmp = gtk_label_new("No preview available");
-	gtk_object_set_data(GTK_OBJECT(ThemeWin), "theme_preview_data", tmp);
+	g_object_set_data(G_OBJECT(ThemeWin), "theme_preview_data", tmp);
 	gtk_container_add(GTK_CONTAINER(Alignment), tmp);
 
-	gtk_object_set_data(GTK_OBJECT(global->ThemeWin), "theme_preview_image", 0);
+	g_object_set_data(G_OBJECT(global->ThemeWin), "theme_preview_image", NULL);
 	str = g_strdup_printf("%d", -1);
-	gtk_object_set_data_full(GTK_OBJECT(global->ThemeWin), "theme_preview_selected", str, g_free);
+	g_object_set_data_full(G_OBJECT(global->ThemeWin), "theme_preview_selected", str, g_free);
 
 	gtk_widget_show_all(ThemeWin);
 }
@@ -388,47 +392,45 @@ void theme_build_selection_win()  {
 void theme_fill_selection_list()  {
 
 	guint32 i;
-	gint32 row;
-	gchar *txt[10];
-	GtkWidget *List;
+	GtkWidget *ThemeList;
+	GtkListStore *store;
 
-	List = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_list");
+	ThemeList = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_list");
 
 	/* build theme list */
-	gtk_clist_freeze(GTK_CLIST(List));
-	gtk_clist_clear(GTK_CLIST(List));
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(ThemeList)));
+	gtk_list_store_clear(store);
 
 	for(i = 0 ; i < MAX_THEMES ; i++)  {
+		GtkTreeIter iter;
 
-		if(data->theme[i].open)  {
-
-			txt[0] = data->theme[i].name;
-			txt[1] = g_strdup_printf("%d", i);
-			row = gtk_clist_append(GTK_CLIST(List), txt);
-			gtk_clist_set_selectable(GTK_CLIST(List), row, TRUE);
-			g_free(txt[1]);
+		if (!data->theme[i].open) {
+			continue;
 		}
-	}
 
-	gtk_clist_thaw(GTK_CLIST(List));
+		gtk_list_store_append(store, &iter);
+		gtk_list_store_set(store, &iter,
+		  THEMELIST_COLUMN_NAME, data->theme[i].name,
+		  THEMELIST_COLUMN_ID, i,
+		  -1);
+	}
 }
 
 
 void theme_preview_free()  {
-
-//	GtkWidget *Alignment;
 	GtkWidget *Preview;
-	gchar *buf = 0;
+	gchar *buf;
 
 	/* free all preview data */
-//	Alignment = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_preview_container");
-	Preview = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_preview_data");
-
+	Preview = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_preview_data");
 	gtk_widget_destroy(Preview);
+	g_object_set_data(G_OBJECT(global->ThemeWin), "theme_preview_data", NULL);
 
-	buf = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_preview_image");
-	if(buf) g_free(buf);
-	gtk_object_set_data(GTK_OBJECT(global->ThemeWin), "theme_preview_image", 0);
+	buf = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_preview_image");
+	if (buf) {
+		g_free(buf);
+		g_object_set_data(G_OBJECT(global->ThemeWin), "theme_preview_image", NULL);
+	}
 }
 
 
@@ -443,112 +445,73 @@ void theme_display(gint32 themeid)  {
 
 	if(! data->theme[themeid].open)  return;
 
-	Label = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_name");
+	Label = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_name");
 	gtk_label_set_text(GTK_LABEL(Label), data->theme[themeid].name);
 
-	Label = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_type");
+	Label = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_type");
 	gtk_label_set_text(GTK_LABEL(Label), data->theme[themeid].type);
 
-	Label = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_version");
+	Label = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_version");
 	gtk_label_set_text(GTK_LABEL(Label), data->theme[themeid].version);
 
-	Label = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_author");
+	Label = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_author");
 	gtk_label_set_text(GTK_LABEL(Label), data->theme[themeid].author);
 
-	Label = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_email");
+	Label = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_email");
 	gtk_label_set_text(GTK_LABEL(Label), data->theme[themeid].email);
 
-	Label = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_location");
+	Label = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_location");
 	gtk_label_set_text(GTK_LABEL(Label), data->theme[themeid].path);
 
 	tmp = g_strdup_printf("%d", themeid);
-	gtk_object_set_data_full(GTK_OBJECT(global->ThemeWin), "theme_preview_selected", tmp, g_free);
-
+	g_object_set_data_full(G_OBJECT(global->ThemeWin), "theme_preview_selected", tmp, g_free);
 
 	/* free all preview data */
-	Alignment = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_preview_container");
-	Preview = gtk_object_get_data(GTK_OBJECT(global->ThemeWin), "theme_preview_data");
-
 	theme_preview_free();
 
+	Alignment = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_preview_container");
+
 	/* put new preview data */
 	if(! (buf = theme_preview_read(themeid, &width, &height)) )  {
 
 		Preview = gtk_label_new("No preview available");
-		gtk_object_set_data(GTK_OBJECT(global->ThemeWin), "theme_preview_data", Preview);
+		g_object_set_data(G_OBJECT(global->ThemeWin), "theme_preview_data", Preview);
 		gtk_container_add(GTK_CONTAINER(Alignment), Preview);
 		gtk_widget_show_all(Alignment);
 		return;
 	}
 
 	/* put preview image */
-	gtk_object_set_data(GTK_OBJECT(global->ThemeWin), "theme_preview_image", buf);
-
-	tmp = g_strdup_printf("%d", width);
-	gtk_object_set_data_full(GTK_OBJECT(global->ThemeWin), "theme_preview_width", tmp, g_free);
-	tmp = g_strdup_printf("%d", height);
-	gtk_object_set_data_full(GTK_OBJECT(global->ThemeWin), "theme_preview_height", tmp, g_free);
+	g_object_set_data(G_OBJECT(global->ThemeWin), "theme_preview_image", buf);
 
 	Preview = gtk_drawing_area_new();
-	gtk_drawing_area_size( GTK_DRAWING_AREA (Preview), width, height);
-	gtk_object_set_data(GTK_OBJECT(global->ThemeWin), "theme_preview_data", Preview);
+	gtk_widget_set_size_request( GTK_WIDGET(Preview), width, height);
+	g_object_set_data(G_OBJECT(global->ThemeWin), "theme_preview_data", Preview);
 	gtk_container_add(GTK_CONTAINER(Alignment), Preview);
-	gtk_signal_connect (GTK_OBJECT (Preview), "expose_event", GTK_SIGNAL_FUNC (on_theme_preview_expose), NULL);
+	g_signal_connect(G_OBJECT (Preview), "draw", G_CALLBACK(on_theme_preview_draw), NULL);
 	gtk_widget_show_all(Alignment);
 }
 
 
 guint8* theme_preview_read(gint32 themeid, guint32 *width, guint32 *height)  {
-
-	struct stat s;
 	gchar *path = data->theme[themeid].path_preview;
-	guint8 *buf;
-
-	png_structp png_ptr;
-	png_infop info_ptr;
+	guint8 *buf = NULL;
 	_png_imagedata png_imagedata;
-	FILE *infile;
-
-
-	if(! data->theme[themeid].open)  return FALSE;
-
-	/* simple test */
-	if(! path)  return FALSE;
-
-	if(stat(path, &s) < 0)  {
-
-		return FALSE;
-	}
 
+	if (!data->theme[themeid].open)  return NULL;
+	if (path == NULL)  return NULL;
 
 	/* load preview image */
-	if (! (infile = fopen(path, "rb")) ) {
-
-		fprintf(stderr, "can't open file [%s]\n", path);
-		return FALSE;
+	if (readpng_read(path, &png_imagedata)) {
+		goto free_and_return;
 	}
 
-	if (! readpng_init(infile, &png_ptr, &info_ptr, &png_imagedata) != 0) {
-
-		fprintf(stderr, "[%s] is not a valid PNG file\n", path);
-		fclose(infile);
-		return FALSE;
-	}
-
-	*width = png_imagedata.width;
-	*height = png_imagedata.height;
-
-	if(! readpng_decode_image(&png_ptr, &info_ptr, &png_imagedata, TRUE) )  {
-
-		fprintf(stderr, "Unable to decode PNG image: [%s]\n", path);
-		return FALSE;
-	}
-	fclose(infile);
-
-	buf = readpng_get_image(&png_imagedata);
-
-	readpng_cleanup(&png_ptr, &info_ptr, &png_imagedata);
+	*width = readpng_get_width(&png_imagedata);
+	*height = readpng_get_height(&png_imagedata);
+	buf = readpng_get_rgba(&png_imagedata);
 
+free_and_return:
+	readpng_cleanup(&png_imagedata);
 	return buf;
 }
 
@@ -558,3 +521,38 @@ void theme_load(gint32 themeid)  {
 	printf("%d\n", themeid);
 	return;
 }
+
+/* when receive event: redraw theme preview */
+void on_theme_preview_draw(GtkWidget *widget, cairo_t *cr, gpointer data) {
+	gint32 height, width, stride, i, j;
+	cairo_surface_t *surface;
+	guint8 *src;
+	guint32 *buf, *dst;
+	(void)data;
+
+	width = gtk_widget_get_allocated_width(widget);
+	height = gtk_widget_get_allocated_height(widget);
+	stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
+
+	/* Convert from RGBA32 big-endian (PNG) to ARGB32 native-endian (Cairo) */
+	src = g_object_get_data(G_OBJECT(global->ThemeWin), "theme_preview_image");
+	buf = dst = g_malloc(stride * height);
+	for(i = 0; i < height ; i++) {
+		for(j = 0; j < width; j++) {
+			guint8 r, g, b;
+			r = *src++;
+			g = *src++;
+			b = *src++;
+			src++; // FIXME: add alpha support, cairo needs premultiplied RGB values
+			// buf = (A << 24) | (R << 16) | (G << 8) | (B);
+			*dst++ = (0xff << 24) | (r << 16) | (g << 8) | b;
+		}
+		dst += stride/4 - width;
+	}
+
+	surface = cairo_image_surface_create_for_data((unsigned char*)buf, CAIRO_FORMAT_ARGB32, width, height, stride);
+	cairo_set_source_surface(cr, surface, 0, 0);
+	cairo_paint(cr);
+	cairo_surface_destroy(surface);
+	g_free(buf);
+}
